# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-18 07:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import getpaid.abstract_mixin


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('getpaid', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EpayPayment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('merchant_id', models.CharField(blank=True, max_length=15)),
                ('card', models.CharField(blank=True, max_length=19)),
                ('reference', models.CharField(blank=True, max_length=15)),
                ('approval_code', models.CharField(blank=True, max_length=15)),
                ('response_code', models.CharField(blank=True, max_length=5)),
                ('is_secure', models.BooleanField(default=True)),
                ('card_bin', models.CharField(blank=True, max_length=10)),
                ('c_hash', models.CharField(blank=True, max_length=36)),
                ('customer_name', models.CharField(blank=True, max_length=50)),
                ('customer_mail', models.CharField(blank=True, max_length=75)),
                ('customer_phone', models.CharField(blank=True, max_length=20)),
                ('is_payment', models.BooleanField(default=True)),
                ('status', models.IntegerField(blank=True, null=True)),
                ('result', models.IntegerField(blank=True, null=True)),
                ('payment', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='getpaid.Payment')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model, getpaid.abstract_mixin.AbstractMixin),
        ),
    ]
